from typing import Tuple, Set, Iterable, List


class IGH_MouseInteraction:
    @property
    def ControlPointDown(self) -> Point: ...
    @property
    def CanvasPointDown(self) -> PointF: ...
    @property
    def DeactivateOnFocusLoss(self) -> bool: ...
    @property
    def IsActive(self) -> bool: ...
    @property
    def Owner(self) -> GH_Canvas: ...
    def Destroy(self) -> None: ...


class GH_AbstractInteraction:
    @property
    def CanvasPointDown(self) -> PointF: ...
    @property
    def ControlPointDown(self) -> Point: ...
    @property
    def Canvas(self) -> GH_Canvas: ...
    @property
    def IsActive(self) -> bool: ...
    @property
    def DeactivateOnFocusLoss(self) -> bool: ...
    def Destroy(self) -> None: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToKeyUp(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToMouseDown(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseDoubleClick(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def IsTooltipRegion(self, canvas_coordinate: PointF) -> bool: ...
    def SetupTooltip(self, canvasPoint: PointF, e: GH_TooltipDisplayEventArgs) -> None: ...
    @property
    def TooltipEnabled(self) -> bool: ...


class GH_WindowSelectInteraction(GH_AbstractInteraction):
    def __init__(self, canvas: GH_Canvas, mEvent: GH_CanvasMouseEvent): ...
    def Destroy(self) -> None: ...
    @property
    def DeactivateOnFocusLoss(self) -> bool: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToKeyUp(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...


class GH_DragInteraction(GH_AbstractInteraction):
    def __init__(self, canvas: GH_Canvas, e: GH_CanvasMouseEvent): ...
    def Destroy(self) -> None: ...
    def AddSnapAttributes(self, attribute: IGH_Attributes) -> None: ...
    def AddAttribute(self, attribute: IGH_Attributes) -> None: ...
    @property
    def AttributeCount(self) -> int: ...
    def RespondToMouseDown(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToKeyUp(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...


class GH_SplitInteraction(GH_AbstractInteraction):
    def __init__(self, canvas: GH_Canvas, e: GH_CanvasMouseEvent): ...
    def Destroy(self) -> None: ...
    def AddAttribute(self, Attribute: IGH_Attributes) -> None: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToKeyUp(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...


class GH_DumpInteraction(GH_AbstractInteraction):
    @overload
    def __init__(self, parent: GH_Canvas): ...
    @overload
    def __init__(self, parent: GH_Canvas, objectId: Guid): ...
    def AppendProxy(self, proxy: IGH_ObjectProxy) -> None: ...
    @property
    def IsAggregating(self) -> bool: ...
    def RespondToMouseDown(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def Destroy(self) -> None: ...


class GH_PanInteraction(GH_AbstractInteraction):
    def __init__(self, iParent: GH_Canvas, mEvent: GH_CanvasMouseEvent): ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...


class GH_ZoomInteraction(GH_AbstractInteraction):
    def __init__(self, iParent: GH_Canvas, mEvent: GH_CanvasMouseEvent): ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...


class GH_CycleInteraction(GH_AbstractInteraction):
    def __init__(self, parentCanvas: GH_Canvas): ...
    def Destroy(self) -> None: ...
    @property
    def DeactivateOnFocusLoss(self) -> bool: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToKeyUp(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToMouseDoubleClick(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseDown(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...


class GH_WireInteraction(GH_AbstractInteraction):
    def __init__(self, iParent: GH_Canvas, mEvent: GH_CanvasMouseEvent, Source: IGH_Param): ...
    def Destroy(self) -> None: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseDown(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToKeyUp(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...


class GH_RewireInteraction(GH_AbstractInteraction):
    def __init__(self, iParent: GH_Canvas, mEvent: GH_CanvasMouseEvent, Source: IGH_Param): ...
    def Destroy(self) -> None: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToKeyUp(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
    @property
    def IsValid(self) -> bool: ...


class GH_RadialMenuInteraction(GH_AbstractInteraction):
    def __init__(self, canvas: GH_Canvas, e: GH_CanvasMouseEvent): ...
    def Destroy(self) -> None: ...
    @property
    def DeactivateOnFocusLoss(self) -> bool: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseDown(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    @property
    def TooltipEnabled(self) -> bool: ...
    def IsTooltipRegion(self, canvas_coordinate: PointF) -> bool: ...
    def SetupTooltip(self, point: PointF, e: GH_TooltipDisplayEventArgs) -> None: ...


class GH_SketchInteraction(GH_AbstractInteraction):
    def __init__(self, iParent: GH_Canvas): ...
    def Destroy(self) -> None: ...
    def RespondToMouseMove(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseDown(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToMouseUp(self, sender: GH_Canvas, e: GH_CanvasMouseEvent) -> GH_ObjectResponse: ...
    def RespondToKeyDown(self, sender: GH_Canvas, e: KeyEventArgs) -> GH_ObjectResponse: ...
