from typing import Tuple, Set, Iterable, List


class GH_LayoutTab:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, name: str): ...
    @overload
    def __init__(self, other: GH_LayoutTab): ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, Value: str) -> None: ...
    @property
    def Selected(self) -> bool: ...
    @Selected.setter
    def Selected(self, Value: bool) -> None: ...
    @property
    def Panels(self) -> List: ...
    def SortPanels(self, order: Set(str)) -> None: ...
    @overload
    def AddPanel(self) -> GH_LayoutPanel: ...
    @overload
    def AddPanel(self, name: str) -> GH_LayoutPanel: ...
    def Deselect(self) -> None: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def CompareTo(self, other: GH_LayoutTab) -> int: ...


class GH_LayoutPanel:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, name: str): ...
    @overload
    def __init__(self, other: GH_LayoutPanel): ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, Value: str) -> None: ...
    @property
    def Items(self) -> List: ...
    def Sort(self) -> None: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def CompareTo(self, other: GH_LayoutPanel) -> int: ...


class GH_LayoutItem:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, objectId: Guid, exposure: GH_Exposure): ...
    @overload
    def __init__(self, other: GH_LayoutItem): ...
    @property
    def Id(self) -> Guid: ...
    @property
    def Selected(self) -> bool: ...
    @Selected.setter
    def Selected(self, Value: bool) -> None: ...
    @property
    def Exposure(self) -> GH_Exposure: ...
    @Exposure.setter
    def Exposure(self, Value: GH_Exposure) -> None: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class GH_Layout:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, other: GH_Layout): ...
    @property
    def Name(self) -> str: ...
    @property
    def FilePath(self) -> str: ...
    @FilePath.setter
    def FilePath(self, Value: str) -> None: ...
    @property
    def Tabs(self) -> List: ...
    def FindTab(self, name: str) -> GH_LayoutTab: ...
    @property
    def TabCount(self) -> int: ...
    @property
    def PanelCount(self) -> int: ...
    @property
    def ItemCount(self) -> int: ...
    @property
    def Items(self) -> Iterable[GH_LayoutItem]: ...
    @overload
    def AddTab(self) -> GH_LayoutTab: ...
    @overload
    def AddTab(self, name: str) -> GH_LayoutTab: ...
    @overload
    def AddItem(self, category: str, subcategory: str, id: Guid) -> None: ...
    @overload
    def AddItem(self, category: str, subcategory: str, id: Guid, exposure: GH_Exposure) -> None: ...
    @overload
    def AddItem(self, category: str, subcategory: str, item: GH_LayoutItem) -> None: ...
    @overload
    def Serialize(self) -> str: ...
    @overload
    def Serialize(self, filepath: str) -> bool: ...
    def Deserialize(self, filepath: str) -> bool: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class GH_LayoutMenuItem(GH_DoubleBufferedPanel):
    def __init__(self): ...
    @property
    def Ribbon(self) -> GH_Ribbon: ...
    @Ribbon.setter
    def Ribbon(self, Value: GH_Ribbon) -> None: ...
    @property
    def LayoutFile(self) -> str: ...
    @LayoutFile.setter
    def LayoutFile(self, Value: str) -> None: ...
    @property
    def VisibleRegion(self) -> Rectangle: ...
    @property
    def CloseRegion(self) -> Rectangle: ...
    @property
    def EditRegion(self) -> Rectangle: ...
    @property
    def LabelRegion(self) -> Rectangle: ...


class GH_RibbonContentBase:
    @property
    def Visible(self) -> bool: ...
    @Visible.setter
    def Visible(self, Value: bool) -> None: ...
    @property
    def Padding(self) -> Padding: ...
    @Padding.setter
    def Padding(self, Value: Padding) -> None: ...
    @property
    def Margin(self) -> Padding: ...
    @Margin.setter
    def Margin(self, Value: Padding) -> None: ...
    def Contains(self, pt: Point) -> bool: ...
    @property
    def ContentRegion(self) -> Rectangle: ...
    @ContentRegion.setter
    def ContentRegion(self, Value: Rectangle) -> None: ...
    @property
    def ClientRegion(self) -> Rectangle: ...
    @ClientRegion.setter
    def ClientRegion(self, Value: Rectangle) -> None: ...
    def PerformLayout(self) -> None: ...


class GH_RibbonItem(GH_RibbonContentBase):
    def __init__(self, item_proxy: IGH_ObjectProxy): ...
    @property
    def Owner(self) -> GH_RibbonPanel: ...
    @Owner.setter
    def Owner(self, Value: GH_RibbonPanel) -> None: ...
    @property
    def Proxy(self) -> IGH_ObjectProxy: ...
    @Proxy.setter
    def Proxy(self, Value: IGH_ObjectProxy) -> None: ...
    def PerformLayout(self) -> None: ...
    def DisplayTooltip(self, e: GH_TooltipDisplayEventArgs) -> None: ...
    def MouseClick(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseDoubleClick(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseDown(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseMove(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseUp(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def RenderItem(self, g: Graphics) -> None: ...
    def CompareTo(self, other: GH_RibbonItem) -> int: ...


class GH_RibbonPanel(GH_RibbonContentBase):
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, iName: str): ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, Value: str) -> None: ...
    @property
    def Owner(self) -> GH_RibbonTab: ...
    @Owner.setter
    def Owner(self, Value: GH_RibbonTab) -> None: ...
    @property
    def AllItems(self) -> List: ...
    @property
    def VisibleItems(self) -> List: ...
    def AddItem(self, item: GH_RibbonItem) -> bool: ...
    def RemoveItem(self, item: GH_RibbonItem) -> bool: ...
    @overload
    def ItemAt(self, index: int) -> GH_RibbonItem: ...
    @overload
    def ItemAt(self, pt: Point) -> GH_RibbonItem: ...
    def IndexAt(self, pt: Point) -> int: ...
    @overload
    def Contains(self, id: Guid, exposure: GH_Exposure) -> bool: ...
    def PanelBarRegion(self) -> Rectangle: ...
    def PerformLayout(self) -> None: ...
    def MoveTo(self, x: int, y: int) -> None: ...
    def CollapseLastColumn(self) -> bool: ...
    def CollapseLeastSignificantColumn(self) -> bool: ...
    def MinimumWidth(self) -> int: ...
    def DesiredHeight(self, initialHeight: int) -> int: ...
    def Sort(self) -> None: ...
    def DisplayTooltip(self, e: GH_TooltipDisplayEventArgs) -> None: ...
    def MouseClick(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseDoubleClick(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseDown(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseMove(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseUp(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    @overload
    def DisplayDropdown(self) -> None: ...
    @overload
    def DisplayDropdown(self, autoCloseDropdown: bool) -> GH_RibbonDropdown: ...
    def RibbonFont(self) -> Font: ...
    def RenderPanel(self, g: Graphics) -> None: ...
    def CompareTo(self, other: GH_RibbonPanel) -> int: ...


class GH_TabDisplay:
    #None = 0
    FullName = 1
    ShortName = 2
    Symbol = 3
    Icon = 4


class GH_RibbonTab(GH_RibbonContentBase):
    def __init__(self, owner: GH_Ribbon, name: str): ...
    @property
    def Owner(self) -> GH_Ribbon: ...
    @property
    def NameFull(self) -> str: ...
    @property
    def NameShort(self) -> str: ...
    @property
    def NameSymbol(self) -> str: ...
    @property
    def Icon(self) -> Bitmap: ...
    @property
    def HasIcon(self) -> bool: ...
    @property
    def DisplayStyle(self) -> GH_TabDisplay: ...
    @DisplayStyle.setter
    def DisplayStyle(self, AutoPropertyValue: GH_TabDisplay) -> None: ...
    @property
    def Grip(self) -> Rectangle: ...
    @Grip.setter
    def Grip(self, AutoPropertyValue: Rectangle) -> None: ...
    @property
    def Panels(self) -> List: ...
    def EnsurePanel(self, name: str) -> GH_RibbonPanel: ...
    def PerformLayout(self) -> None: ...
    def DisplayTooltip(self, e: GH_TooltipDisplayEventArgs) -> None: ...
    def MouseClick(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseDoubleClick(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseDown(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseMove(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseUp(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def RenderTab(self, g: Graphics) -> None: ...


class GH_Ribbon:
    def __init__(self): ...
    @property
    def ActiveTabIndex(self) -> int: ...
    @ActiveTabIndex.setter
    def ActiveTabIndex(self, Value: int) -> None: ...
    @property
    def ActiveTabName(self) -> str: ...
    @ActiveTabName.setter
    def ActiveTabName(self, Value: str) -> None: ...
    @property
    def ActiveTab(self) -> GH_RibbonTab: ...
    @ActiveTab.setter
    def ActiveTab(self, Value: GH_RibbonTab) -> None: ...
    @property
    def Tabs(self) -> List: ...
    def PopulateRibbon(self) -> None: ...
    def LayoutRibbon(self) -> None: ...
    def NearestHeight(self, initialHeight: int) -> int: ...
    def NearestWidth(self, w: int) -> int: ...
    def RenderFocusRectangle(self, g: Graphics) -> None: ...
    def AmIFocused(self, item: GH_RibbonItem) -> bool: ...
    @property
    def TooltipDelay(self) -> int: ...
    @TooltipDelay.setter
    def TooltipDelay(self, Value: int) -> None: ...
    @property
    def ActiveObject(self) -> IGH_RibbonInteractiveObject: ...
    @ActiveObject.setter
    def ActiveObject(self, Value: IGH_RibbonInteractiveObject) -> None: ...
    @property
    def IsActiveObject(self) -> bool: ...
    def CreateRibbonLayoutMenu(self) -> ToolStripDropDown: ...
    def FindAndDisplayProxy(self, id: Guid) -> Tuple[bool, Rectangle, Rectangle, GH_RibbonDropdown]: ...


class GH_RibbonPainter:
    def TabPaneBrush(rec: Rectangle, bg: Color) -> Brush: ...
    def TabEdgeBrush(rec: Rectangle) -> Brush: ...
    def PanelOuterBorder(rec: Rectangle) -> GraphicsPath: ...
    def PanelInnerBorder(rec: Rectangle) -> GraphicsPath: ...
    def PanelBottomBar(rec: Rectangle) -> GraphicsPath: ...
    def DropDownBar(rec: Rectangle) -> GraphicsPath: ...
    @property
    def PanelBarHeight() -> int: ...
    @property
    def PanelCornerRadius() -> int: ...
    @property
    def TabGripHeight() -> int: ...
    def TabOuterBorder(GripRec: Rectangle, ContentRec: Rectangle) -> GraphicsPath: ...


class GH_RibbonDropdown:
    def __init__(self): ...
    def SetOwner(self, owner: GH_RibbonPanel) -> None: ...
    @property
    def AutoClose(self) -> bool: ...
    @AutoClose.setter
    def AutoClose(self, Value: bool) -> None: ...
    def PopulateContent(self, proxies: List) -> None: ...
    def FindIconRectangle(self, id: Guid) -> Tuple[bool, Rectangle]: ...


class GH_RibbonMouseEvent:
    Unset = 0
    Ignored = 1
    Handled = 2
    Handled_Redraw = 3


class GH_RibbonMouseEventArgs:
    @overload
    def __init__(self, e: MouseEventArgs): ...
    @overload
    def __init__(self, e: MouseEventArgs, nActiveObject: IGH_RibbonInteractiveObject): ...
    def Reset(self) -> None: ...
    @property
    def ActiveObject(self) -> IGH_RibbonInteractiveObject: ...
    @property
    def IsActiveObject(self) -> bool: ...
    @property
    def Release(self) -> bool: ...
    def NewActiveObject(self, nObject: IGH_RibbonInteractiveObject) -> None: ...
    @overload
    def ReleaseActiveObject(self) -> None: ...
    @overload
    def ReleaseActiveObject(self, owner_filter: IGH_RibbonInteractiveObject) -> None: ...


class IGH_RibbonInteractiveObject:
    def MouseDown(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseMove(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseUp(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseClick(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def MouseDoubleClick(self, sender: GH_Ribbon, e: GH_RibbonMouseEventArgs) -> GH_RibbonMouseEvent: ...
    def DisplayTooltip(self, e: GH_TooltipDisplayEventArgs) -> None: ...
