from typing import Tuple, Set, Iterable, List


class ITransform:
    @property
    def Name(self) -> str: ...
    def Reverse(self) -> ITransform: ...
    def ToMatrix(self) -> Transform: ...
    def ToString(self) -> str: ...
    def Duplicate(self) -> ITransform: ...


class Orientation:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, source: Plane, target: Plane): ...
    def Duplicate(self) -> ITransform: ...
    def Reverse(self) -> ITransform: ...
    @property
    def SourceFrame(self) -> Plane: ...
    @SourceFrame.setter
    def SourceFrame(self, Value: Plane) -> None: ...
    @property
    def TargetFrame(self) -> Plane: ...
    @TargetFrame.setter
    def TargetFrame(self, Value: Plane) -> None: ...
    @property
    def Name(self) -> str: ...
    def ToMatrix(self) -> Transform: ...
    def ToString(self) -> str: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class Translation:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, translation: Vector3d): ...
    def Duplicate(self) -> ITransform: ...
    def Reverse(self) -> ITransform: ...
    @property
    def TranslationVector(self) -> Vector3d: ...
    @TranslationVector.setter
    def TranslationVector(self, Value: Vector3d) -> None: ...
    @property
    def Name(self) -> str: ...
    def ToMatrix(self) -> Transform: ...
    def ToString(self) -> str: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class Projection:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, plane: Plane): ...
    def Duplicate(self) -> ITransform: ...
    def Reverse(self) -> ITransform: ...
    @property
    def ProjectionPlane(self) -> Plane: ...
    @ProjectionPlane.setter
    def ProjectionPlane(self, Value: Plane) -> None: ...
    @property
    def Name(self) -> str: ...
    def ToMatrix(self) -> Transform: ...
    def ToString(self) -> str: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class Rotation:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, center: Point3d, axis: Vector3d, angle: float): ...
    @overload
    def __init__(self, center: Point3d, dir0: Vector3d, dir1: Vector3d): ...
    def Duplicate(self) -> ITransform: ...
    def Reverse(self) -> ITransform: ...
    @property
    def RotationCenter(self) -> Point3d: ...
    @RotationCenter.setter
    def RotationCenter(self, Value: Point3d) -> None: ...
    @property
    def RotationAxis(self) -> Vector3d: ...
    @RotationAxis.setter
    def RotationAxis(self, Value: Vector3d) -> None: ...
    @property
    def RotationAngle(self) -> float: ...
    @RotationAngle.setter
    def RotationAngle(self, Value: float) -> None: ...
    @property
    def Name(self) -> str: ...
    def ToMatrix(self) -> Transform: ...
    def ToString(self) -> str: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class Generic:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, transform: Transform): ...
    def Duplicate(self) -> ITransform: ...
    def Reverse(self) -> ITransform: ...
    @property
    def Transform(self) -> Transform: ...
    @Transform.setter
    def Transform(self, Value: Transform) -> None: ...
    @property
    def Name(self) -> str: ...
    def ToMatrix(self) -> Transform: ...
    def ToString(self) -> str: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class Scale:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, point: Point3d, scale: float): ...
    @overload
    def __init__(self, frame: Plane, scale: float): ...
    @overload
    def __init__(self, frame: Plane, xscale: float, yscale: float, zscale: float): ...
    def Duplicate(self) -> ITransform: ...
    def Reverse(self) -> ITransform: ...
    @property
    def ScalingFrame(self) -> Plane: ...
    @ScalingFrame.setter
    def ScalingFrame(self, Value: Plane) -> None: ...
    @property
    def ScaleXFactor(self) -> float: ...
    @ScaleXFactor.setter
    def ScaleXFactor(self, Value: float) -> None: ...
    @property
    def ScaleYFactor(self) -> float: ...
    @ScaleYFactor.setter
    def ScaleYFactor(self, Value: float) -> None: ...
    @property
    def ScaleZFactor(self) -> float: ...
    @ScaleZFactor.setter
    def ScaleZFactor(self, Value: float) -> None: ...
    @property
    def Name(self) -> str: ...
    def ToMatrix(self) -> Transform: ...
    def ToString(self) -> str: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class Identity:
    def __init__(self): ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def Duplicate(self) -> ITransform: ...
    @property
    def Name(self) -> str: ...
    def ToString(self) -> str: ...
    def Reverse(self) -> ITransform: ...
    def ToMatrix(self) -> Transform: ...
