from typing import Tuple, Set, Iterable, List


class GH_DifferentialSolver:
    #None = 0
    Euler = 1
    RungeKutta2 = 2
    RungeKutta3 = 3
    RungeKutta4 = 4


class GH_Field:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, other: GH_Field): ...
    def Duplicate(self) -> IGH_Goo: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    @property
    def Elements(self) -> List: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self, target: T) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @overload
    def ToString(self) -> str: ...
    def TensorAt(self, location: Point3d) -> Vector3d: ...
    def CurvatureAt(self, location: Point3d) -> float: ...
    def SolveStep(self, location: Point3d, factor: float, method: GH_DifferentialSolver) -> Vector3d: ...
    def SolveSteps(self, location: Point3d, accuracy: float, count: int, method: GH_DifferentialSolver) -> Point3dList: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...


class IGH_FieldElement:
    def Duplicate(self) -> IGH_FieldElement: ...
    def Force(self, location: Point3d) -> Vector3d: ...
    @property
    def ElementGuid(self) -> Guid: ...
    @property
    def BoundingBox(self) -> BoundingBox: ...
    @property
    def IsLimited(self) -> bool: ...
    @IsLimited.setter
    def IsLimited(self, Value: bool) -> None: ...
    @property
    def Limits(self) -> Box: ...
    @Limits.setter
    def Limits(self, Value: Box) -> None: ...
    def IsCoincident(self, point: Point3d, tolerance: float) -> bool: ...


class GH_FieldElement:
    @property
    def ElementGuid(self) -> Guid: ...
    @property
    def BoundingBox(self) -> BoundingBox: ...
    @property
    def IsLimited(self) -> bool: ...
    @IsLimited.setter
    def IsLimited(self, Value: bool) -> None: ...
    @property
    def Limits(self) -> Box: ...
    @Limits.setter
    def Limits(self, Value: Box) -> None: ...
    def Duplicate(self) -> IGH_FieldElement: ...
    def Force(self, location: Point3d) -> Vector3d: ...
    def IsCoincident(self, point: Point3d, tolerance: float) -> bool: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...


class GH_PointCharge(GH_FieldElement):
    def __init__(self): ...
    def Duplicate(self) -> IGH_FieldElement: ...
    @property
    def Location(self) -> Point3d: ...
    @Location.setter
    def Location(self, Value: Point3d) -> None: ...
    @property
    def Charge(self) -> float: ...
    @Charge.setter
    def Charge(self, Value: float) -> None: ...
    @property
    def Decay(self) -> float: ...
    @Decay.setter
    def Decay(self, Value: float) -> None: ...
    @property
    def ElementGuid(self) -> Guid: ...
    @property
    def BoundingBox(self) -> BoundingBox: ...
    def IsCoincident(self, point: Point3d, tolerance: float) -> bool: ...
    def Force(self, location: Point3d) -> Vector3d: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...


class GH_LineCharge(GH_FieldElement):
    def __init__(self): ...
    def Duplicate(self) -> IGH_FieldElement: ...
    @property
    def Segment(self) -> Line: ...
    @Segment.setter
    def Segment(self, Value: Line) -> None: ...
    @property
    def Charge(self) -> float: ...
    @Charge.setter
    def Charge(self, Value: float) -> None: ...
    @property
    def ElementGuid(self) -> Guid: ...
    @property
    def BoundingBox(self) -> BoundingBox: ...
    def IsCoincident(self, point: Point3d, tolerance: float) -> bool: ...
    def Force(self, location: Point3d) -> Vector3d: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...


class GH_VectorForce(GH_FieldElement):
    def __init__(self): ...
    def Duplicate(self) -> IGH_FieldElement: ...
    @property
    def Chord(self) -> Line: ...
    @Chord.setter
    def Chord(self, Value: Line) -> None: ...
    @property
    def ElementGuid(self) -> Guid: ...
    @property
    def BoundingBox(self) -> BoundingBox: ...
    def Force(self, location: Point3d) -> Vector3d: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...


class GH_SpinForce(GH_FieldElement):
    def __init__(self): ...
    def Duplicate(self) -> IGH_FieldElement: ...
    @property
    def Plane(self) -> Plane: ...
    @Plane.setter
    def Plane(self, Value: Plane) -> None: ...
    @property
    def Radius(self) -> float: ...
    @Radius.setter
    def Radius(self, Value: float) -> None: ...
    @property
    def Strength(self) -> float: ...
    @Strength.setter
    def Strength(self, Value: float) -> None: ...
    @property
    def Decay(self) -> float: ...
    @Decay.setter
    def Decay(self, Value: float) -> None: ...
    @property
    def ElementGuid(self) -> Guid: ...
    @property
    def BoundingBox(self) -> BoundingBox: ...
    def Force(self, location: Point3d) -> Vector3d: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...


class GH_Culture:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, culture: CultureInfo): ...
    @overload
    def __init__(self, other: GH_Culture): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateCulture(self) -> GH_Culture: ...
    @property
    def IsValid(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def ScriptVariable(self) -> Object: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_LonLatCoordinate:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, longitude: float, latitude: float): ...
    def Duplicate(self) -> IGH_Goo: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    @property
    def Latitude(self) -> float: ...
    @Latitude.setter
    def Latitude(self, Value: float) -> None: ...
    @property
    def Longitude(self) -> float: ...
    @Longitude.setter
    def Longitude(self, Value: float) -> None: ...
    @property
    def SphereUParameter(self) -> float: ...
    @property
    def SphereVParameter(self) -> float: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    def ScriptVariable(self) -> Object: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self, target: T) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class GH_Matrix:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, matrix: Matrix): ...
    @overload
    def __init__(self, other: GH_Matrix): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateMatrix(self) -> GH_Matrix: ...
    def CloneMatrix(matrix: Matrix) -> Matrix: ...
    @property
    def IsValid(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def ScriptVariable(self) -> Object: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...


class GH_SubD:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, subd: SubD): ...
    @overload
    def __init__(self, id: Guid): ...
    @overload
    def __init__(self, other: GH_SubD): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateSubD(self) -> GH_SubD: ...
    def ClearCaches(self) -> None: ...
    @property
    def Value(self) -> SubD: ...
    @Value.setter
    def Value(self, Value: SubD) -> None: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def GetPreviewMeshes(self) -> Set(Mesh): ...
    def DestroyPreviewMeshes(self) -> None: ...
    def AppendRenderGeometry(self, args: GH_RenderArgs, materialHint: RenderMaterial) -> None: ...


class GH_GeometryGroup:
    def __init__(self): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    @property
    def Objects(self) -> List: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    @property
    def IsReferencedGeometry(self) -> bool: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    def ToString(self) -> str: ...
    def ClearCaches(self) -> None: ...
    @overload
    def LoadGeometry(self) -> bool: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def ScriptVariable(self) -> Object: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self, target: T) -> Tuple[bool, T]: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    def AppendRenderGeometry(self, args: GH_RenderArgs, material: RenderMaterial) -> None: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class GH_Transform:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, transform: ITransform): ...
    @overload
    def __init__(self, transform: Transform): ...
    @overload
    def __init__(self, other: GH_Transform): ...
    def Duplicate(self) -> IGH_Goo: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def Value(self) -> Transform: ...
    @Value.setter
    def Value(self, Value: Transform) -> None: ...
    @property
    def CompoundTransforms(self) -> List: ...
    def ClearCaches(self) -> None: ...
    def GetInverse(self) -> GH_Transform: ...
    def ToString(self) -> str: ...
    def ScriptVariable(self) -> Object: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self, target: Q) -> Tuple[bool, Q]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Arc:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, arc: Arc): ...
    @overload
    def __init__(self, other: GH_Arc): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateArc(self) -> GH_Arc: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Boolean:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, bool: bool): ...
    @overload
    def __init__(self, other: GH_Boolean): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateBoolean(self) -> GH_Boolean: ...
    @property
    def IsValid(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Box:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, box: Box): ...
    @overload
    def __init__(self, box: BoundingBox): ...
    @overload
    def __init__(self, id: Guid): ...
    @overload
    def __init__(self, other: GH_Box): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateBox(self) -> GH_Box: ...
    @property
    def Value(self) -> Box: ...
    @Value.setter
    def Value(self, Value: Box) -> None: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def ClearCaches(self) -> None: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def Brep(self) -> Brep: ...
    def Mesh(self) -> Mesh: ...
    def Geometry(self, brp: Brep, crv: Curve, pt: Point) -> Tuple[bool, Brep, Curve, Point]: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def AppendRenderGeometry(self, args: GH_RenderArgs, materialHint: RenderMaterial) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Brep:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, brep: Brep): ...
    @overload
    def __init__(self, id: Guid): ...
    @overload
    def __init__(self, other: GH_Brep): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateBrep(self) -> GH_Brep: ...
    def ClearCaches(self) -> None: ...
    @property
    def Value(self) -> Brep: ...
    @Value.setter
    def Value(self, Value: Brep) -> None: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def BrepTightBoundingBox(in: Brep) -> BoundingBox: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def GetPreviewMeshes(self) -> Set(Mesh): ...
    def DestroyPreviewMeshes(self) -> None: ...
    def AppendRenderGeometry(self, args: GH_RenderArgs, materialHint: RenderMaterial) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Circle:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, circle: Circle): ...
    @overload
    def __init__(self, other: GH_Circle): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateCircle(self) -> GH_Circle: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Colour:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, colour: Color): ...
    @overload
    def __init__(self, other: GH_Colour): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateColour(self) -> GH_Colour: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class Complex:
    @overload
    def __init__(self, real_component: float): ...
    @overload
    def __init__(self, real_component: float, imaginary_component: float): ...
    def GetHashCode(self) -> int: ...
    @property
    def NaN() -> Complex: ...
    @property
    def Zero() -> Complex: ...
    @property
    def ComplexUnit() -> Complex: ...
    @property
    def Infinity() -> Complex: ...
    @property
    def NegativeInfinity() -> Complex: ...
    def op_UnaryPlus(summand: Complex) -> Complex: ...
    def op_UnaryNegation(subtrahend: Complex) -> Complex: ...
    @overload
    def op_Addition(A: Complex, B: Complex) -> Complex: ...
    @overload
    def op_Subtraction(A: Complex, B: Complex) -> Complex: ...
    @overload
    def op_Addition(A: Complex, B: float) -> Complex: ...
    @overload
    def op_Subtraction(A: Complex, B: float) -> Complex: ...
    @overload
    def op_Addition(A: float, B: Complex) -> Complex: ...
    @overload
    def op_Subtraction(A: float, B: Complex) -> Complex: ...
    @overload
    def op_Multiply(A: Complex, B: Complex) -> Complex: ...
    @overload
    def op_Multiply(A: Complex, B: float) -> Complex: ...
    @overload
    def op_Multiply(A: float, B: Complex) -> Complex: ...
    @overload
    def op_Division(dividend: Complex, divisor: Complex) -> Complex: ...
    @overload
    def op_Division(dividend: Complex, divisor: float) -> Complex: ...
    @overload
    def op_Division(dividend: float, divisor: Complex) -> Complex: ...
    @property
    def Real(self) -> float: ...
    @Real.setter
    def Real(self, Value: float) -> None: ...
    @property
    def Imaginary(self) -> float: ...
    @Imaginary.setter
    def Imaginary(self, Value: float) -> None: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsZero(self) -> bool: ...
    @property
    def IsReal(self) -> bool: ...
    @property
    def IsRealNonNegative(self) -> bool: ...
    @property
    def Argument(self) -> float: ...
    @Argument.setter
    def Argument(self, Value: float) -> None: ...
    def Modulus(self) -> float: ...
    def ModulusSquared(self) -> float: ...
    def Sin(self) -> Complex: ...
    def ASin(self) -> Complex: ...
    def Secant(self) -> Complex: ...
    def Cos(self) -> Complex: ...
    def ACos(self) -> Complex: ...
    def Cosecant(self) -> Complex: ...
    def Tan(self) -> Complex: ...
    def ATan(self) -> Complex: ...
    def CoTan(self) -> Complex: ...
    def Exponential(self) -> Complex: ...
    def Log(self) -> Complex: ...
    def Log10(self) -> Complex: ...
    def Power(self, exponent: Complex) -> Complex: ...
    def Root(self, rootexponent: Complex) -> Complex: ...
    def Square(self) -> Complex: ...
    def SquareRoot(self) -> Complex: ...


class GH_ComplexNumber:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, number: Complex): ...
    @overload
    def __init__(self, other: GH_ComplexNumber): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateComplex(self) -> GH_ComplexNumber: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Curve:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, curve: Curve): ...
    @overload
    def __init__(self, ref_guid: Guid): ...
    @overload
    def __init__(self, ref_guid: Guid, ref_edge: int): ...
    @overload
    def __init__(self, other: GH_Curve): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateCurve(self) -> GH_Curve: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    @property
    def Value(self) -> Curve: ...
    @Value.setter
    def Value(self, Value: Curve) -> None: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @property
    def ReferenceEdge(self) -> int: ...
    @ReferenceEdge.setter
    def ReferenceEdge(self, Value: int) -> None: ...
    def ClearCaches(self) -> None: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def MakeDeformable(self) -> bool: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Guid:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, id: Guid): ...
    @overload
    def __init__(self, other: GH_Guid): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGuid(self) -> GH_Guid: ...
    @property
    def IsValid(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Integer:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, number: int): ...
    @overload
    def __init__(self, other: GH_Integer): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateInteger(self) -> GH_Integer: ...
    @property
    def IsValid(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Interval:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, interval: Interval): ...
    @overload
    def __init__(self, other: GH_Interval): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateInterval(self) -> GH_Interval: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def ToString(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def QC_Bool(self) -> bool: ...
    def QC_Col(self) -> Color: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Text(self) -> str: ...
    def QC_Interval(self) -> Interval: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class UVInterval:
    def __init__(self, newU: Interval, newV: Interval): ...
    @property
    def U(self) -> Interval: ...
    @U.setter
    def U(self, Value: Interval) -> None: ...
    @property
    def V(self) -> Interval: ...
    @V.setter
    def V(self, Value: Interval) -> None: ...
    @property
    def U0(self) -> float: ...
    @U0.setter
    def U0(self, Value: float) -> None: ...
    @property
    def U1(self) -> float: ...
    @U1.setter
    def U1(self, Value: float) -> None: ...
    @property
    def V0(self) -> float: ...
    @V0.setter
    def V0(self, Value: float) -> None: ...
    @property
    def V1(self) -> float: ...
    @V1.setter
    def V1(self, Value: float) -> None: ...
    @property
    def IsValid(self) -> bool: ...


class GH_Interval2D:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, uv: UVInterval): ...
    @overload
    def __init__(self, other: GH_Interval2D): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateInterval(self) -> GH_Interval2D: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self, target: T) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Line:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, line: Line): ...
    @overload
    def __init__(self, other: GH_Line): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateLine(self) -> GH_Line: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_MeshingParameters:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, other: MeshingParameters): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateMesherSettings(self) -> GH_MeshingParameters: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def ToString(self) -> str: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class GH_Rectangle:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, rec: Rectangle3d): ...
    @overload
    def __init__(self, other: GH_Rectangle): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateRectangle(self) -> GH_Rectangle: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Mesh:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, mesh: Mesh): ...
    @overload
    def __init__(self, id: Guid): ...
    @overload
    def __init__(self, other: GH_Mesh): ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateMesh(self) -> GH_Mesh: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    def ClearCaches(self) -> None: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def AppendRenderGeometry(self, args: GH_RenderArgs, materialHint: RenderMaterial) -> None: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_MeshFace:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, nA: int, nB: int, nC: int): ...
    @overload
    def __init__(self, nA: int, nB: int, nC: int, nD: int): ...
    @overload
    def __init__(self, other: MeshFace): ...
    @overload
    def __init__(self, other: GH_MeshFace): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateMeshFace(self) -> GH_MeshFace: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def ToString(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self, target: T) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class GH_Number:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, number: float): ...
    @overload
    def __init__(self, other: GH_Number): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateNumber(self) -> GH_Number: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Plane:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, plane: Plane): ...
    @overload
    def __init__(self, id: Guid): ...
    @overload
    def __init__(self, other: GH_Plane): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicatePlane(self) -> GH_Plane: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    @overload
    def DrawPlane(display: DisplayPipeline, plane: Plane) -> None: ...
    @overload
    def DrawPlane(display: DisplayPipeline, plane: Plane, size: float, frequency: int) -> None: ...
    @overload
    def DrawPlane(display: DisplayPipeline, plane: Plane, size: float, frequency: int, grid_color: Color, x_color: Color, y_color: Color) -> None: ...
    @overload
    def DrawPlane(display: DisplayPipeline, plane: Plane, size: float, frequency: int, grid_color: Color, x_color: Color, y_color: Color, back_color: Color) -> None: ...
    def DrawPlaneIcon(display: DisplayPipeline, plane: Plane, size: float, edgeColour: Color, fillColour: Color) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Material:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, rdkId: Guid): ...
    @overload
    def __init__(self, rdkXml: str): ...
    @overload
    def __init__(self, diffuse: Color): ...
    @overload
    def __init__(self, material: DisplayMaterial): ...
    @overload
    def __init__(self, material: RenderMaterial): ...
    @overload
    def __init__(self, other: GH_Material): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateMaterial(self) -> GH_Material: ...
    def CreateStandardMaterial(colour: Color) -> DisplayMaterial: ...
    def CreateRhinoMaterial(colour: Color) -> Material: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def Type(self) -> MaterialType: ...
    @property
    def RdkMaterialId(self) -> Guid: ...
    @RdkMaterialId.setter
    def RdkMaterialId(self, AutoPropertyValue: Guid) -> None: ...
    @property
    def RdkMaterialXml(self) -> str: ...
    @RdkMaterialXml.setter
    def RdkMaterialXml(self, AutoPropertyValue: str) -> None: ...
    @property
    def RdkMaterialRmtl(self) -> str: ...
    @RdkMaterialRmtl.setter
    def RdkMaterialRmtl(self, AutoPropertyValue: str) -> None: ...
    def ToString(self) -> str: ...
    def ScriptVariable(self) -> Object: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self, target: T) -> Tuple[bool, T]: ...
    def MaterialBestGuess(self) -> RenderMaterial: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_String:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, str: str): ...
    @overload
    def __init__(self, other: GH_String): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateString(self) -> GH_String: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class IGH_GeometricGoo:
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @property
    def IsReferencedGeometry(self) -> bool: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    @overload
    def LoadGeometry(self) -> bool: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def ClearCaches(self) -> None: ...




class GH_GeometricGooWrapper:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, goo: IGH_GeometricGoo): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    @property
    def InternalGoo(self) -> IGH_GeometricGoo: ...
    def ToString(self) -> str: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    def ClearCaches(self) -> None: ...
    @overload
    def LoadGeometry(self) -> bool: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    @property
    def IsReferencedGeometry(self) -> bool: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self, target: T) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...


class IGH_GooProxy:
    @property
    def ProxyOwner(self) -> IGH_Goo: ...
    @property
    def Valid(self) -> bool: ...
    @property
    def UserString(self) -> str: ...
    @UserString.setter
    def UserString(self, Value: str) -> None: ...
    @property
    def IsParsable(self) -> bool: ...
    def FormatInstance(self) -> str: ...
    def FromString(self, in: str) -> bool: ...
    def MutateString(self, in: str) -> str: ...
    def Construct(self) -> None: ...




class GH_PointUtil:
    def FitPlaneThroughPoints(pts: Iterable[GH_Point]) -> Plane: ...
    def PullPointsToPlane(pts: Iterable[GH_Point], plane: Plane, include_nulls: bool) -> List: ...
    def ProjectPointsToPlane(pts: Iterable[GH_Point], plane: Plane, include_nulls: bool) -> List: ...
    def RemapPointsToPlane(pts: Iterable[GH_Point], plane: Plane, include_nulls: bool) -> List: ...


class GH_StructurePath:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, path: GH_Path): ...
    @overload
    def __init__(self, other: GH_StructurePath): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicatePath(self) -> GH_StructurePath: ...
    @property
    def Value(self) -> GH_Path: ...
    @Value.setter
    def Value(self, Value: GH_Path) -> None: ...
    @property
    def IsValid(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def ScriptVariable(self) -> Object: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Surface:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, srf: Surface): ...
    @overload
    def __init__(self, brep: Brep): ...
    @overload
    def __init__(self, id: Guid): ...
    @overload
    def __init__(self, other: GH_Surface): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicateSurface(self) -> GH_Surface: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    @property
    def Face(self) -> BrepFace: ...
    @property
    def Value(self) -> Brep: ...
    @Value.setter
    def Value(self, new_value: Brep) -> None: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    def ClearCaches(self) -> None: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    def Untrim(self) -> bool: ...
    def IsPointOnFace(self, u: float, v: float) -> bool: ...
    def IsPointOnDomain(self, u: float, v: float) -> bool: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def GetPreviewMeshes(self) -> Set(Mesh): ...
    def DestroyPreviewMeshes(self) -> None: ...
    def AppendRenderGeometry(self, args: GH_RenderArgs, materialHint: RenderMaterial) -> None: ...
    def ScriptVariable(self) -> Object: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Time:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, time: DateTime): ...
    @overload
    def __init__(self, other: GH_Time): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateDateTime(self) -> GH_Time: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def EmitProxy(self) -> IGH_GooProxy: ...


class GH_Point:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, pt: Point3d): ...
    @overload
    def __init__(self, iOther: GH_Point): ...
    @overload
    def __init__(self, rh_obj_id: Guid): ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateGeometry(self) -> IGH_GeometricGoo: ...
    def DuplicatePoint(self) -> GH_Point: ...
    def CreateFromCoordinate(self, pt: Point3d) -> None: ...
    def CreateFromPointObject(self, id: Guid) -> None: ...
    def CreateFromCurveRatio(self, crv_id: Guid, crv: Curve, t: float) -> None: ...
    def CreateFromCurveDistance(self, crv_id: Guid, crv: Curve, t: float, bFromStart: bool) -> None: ...
    def CreateFromEdgeRatio(self, brp_id: Guid, edge: Curve, e_index: int, t: float) -> None: ...
    def CreateFromEdgeDistance(self, brp_id: Guid, edge: Curve, e_index: int, t: float, bFromStart: bool) -> None: ...
    def CreateFromSurfaceParam(self, brp_id: Guid, f_index: int, srf: Surface, u: float, v: float) -> None: ...
    @property
    def Value(self) -> Point3d: ...
    @Value.setter
    def Value(self, new_value: Point3d) -> None: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @overload
    def ShowReferenceDialog(self) -> DialogResult: ...
    @overload
    def ShowReferenceDialog(self, owner: IWin32Window) -> DialogResult: ...
    def EnsureReferenceData(self) -> None: ...
    @property
    def ReferenceData(self) -> GH_PointRefData: ...
    @ReferenceData.setter
    def ReferenceData(self, Value: GH_PointRefData) -> None: ...
    @overload
    def ReferenceIndex(self) -> int: ...
    @overload
    def ReferenceIndex(self, new_index: int) -> None: ...
    @overload
    def ReferenceParam(self, index: int) -> float: ...
    @overload
    def ReferenceParam(self, index: int, new_param: float) -> None: ...
    def SetReferenceParams(self, u: float, v: float) -> None: ...
    def ReferencesEdge(self) -> bool: ...
    def ReferencesCurve(self) -> bool: ...
    @overload
    def ReferenceType(self) -> GH_PointRefType: ...
    @overload
    def ReferenceType(self, new_type: GH_PointRefType) -> None: ...
    @property
    def IsReferencedGeometry(self) -> bool: ...
    @property
    def ReferenceID(self) -> Guid: ...
    @ReferenceID.setter
    def ReferenceID(self, Value: Guid) -> None: ...
    @property
    def IsGeometryLoaded(self) -> bool: ...
    @overload
    def LoadGeometry(self) -> bool: ...
    @overload
    def LoadGeometry(self, doc: RhinoDoc) -> bool: ...
    @overload
    def ReferenceCurve(self) -> Curve: ...
    @overload
    def ReferenceCurve(self, ref: RhinoObject) -> Curve: ...
    def ReferenceSurface(self) -> BrepFace: ...
    def ClearCaches(self) -> None: ...
    @property
    def Boundingbox(self) -> BoundingBox: ...
    def GetBoundingBox(self, xform: Transform) -> BoundingBox: ...
    def Transform(self, xform: Transform) -> IGH_GeometricGoo: ...
    def Morph(self, xmorph: SpaceMorph) -> IGH_GeometricGoo: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...


class GH_Vector:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, vector: Vector3d): ...
    @overload
    def __init__(self, other: GH_Vector): ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateVector(self) -> GH_Vector: ...
    @property
    def IsValid(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Hash(self) -> int: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Bool(self) -> bool: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Col(self) -> Color: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...
    def Write(self, writer: GH_IWriter) -> bool: ...
    def Read(self, reader: GH_IReader) -> bool: ...




class GH_ObjectWrapper:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, obj: Object): ...
    def Duplicate(self) -> IGH_Goo: ...
    def DuplicateObject(self) -> GH_ObjectWrapper: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...
    def BakeGeometry(self, doc: RhinoDoc, att: ObjectAttributes, obj_guid: Guid) -> Tuple[bool, Guid]: ...
    @property
    def ClippingBox(self) -> BoundingBox: ...
    def DrawViewportMeshes(self, args: GH_PreviewMeshArgs) -> None: ...
    def DrawViewportWires(self, args: GH_PreviewWireArgs) -> None: ...
    @property
    def IsValid(self) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    def Read(self, reader: GH_IReader) -> bool: ...


class GH_PointRefType:
    coordinate = 1
    point_object = 2
    curve_ratio = 10
    curve_dist_start = 11
    curve_dist_end = 12
    srf_param = 20


class GH_PointRefData:
    @overload
    def __init__(self): ...
    @overload
    def __init__(self, iOther: GH_PointRefData): ...
    def SetCurveParam(self, c: Curve, t: float) -> bool: ...
    def SetSurfaceParam(self, srf: Surface, u: float, v: float) -> bool: ...
    def EvCurveParam(self, c: Curve) -> float: ...
    def EvCurve(self, c: Curve) -> Point3d: ...
    def EvSurface(self, s: Surface) -> Point3d: ...


class IGH_Goo:
    def Duplicate(self) -> IGH_Goo: ...
    def ToString(self) -> str: ...
    @property
    def IsValid(self) -> bool: ...
    @property
    def IsValidWhyNot(self) -> str: ...
    @property
    def TypeName(self) -> str: ...
    @property
    def TypeDescription(self) -> str: ...
    def EmitProxy(self) -> IGH_GooProxy: ...
    def CastFrom(self, source: Object) -> bool: ...
    def CastTo(self) -> Tuple[bool, T]: ...
    def ScriptVariable(self) -> Object: ...




class GH_QuickCastType:
    bool = 0
    int = 1
    num = 2
    text = 3
    col = 4
    pt = 5
    vec = 6
    complex = 7
    interval = 8
    matrix = 9


class IGH_QuickCast:
    @property
    def QC_Type(self) -> GH_QuickCastType: ...
    def QC_Distance(self, other: IGH_QuickCast) -> float: ...
    def QC_Hash(self) -> int: ...
    def QC_Bool(self) -> bool: ...
    def QC_Int(self) -> int: ...
    def QC_Num(self) -> float: ...
    def QC_Text(self) -> str: ...
    def QC_Col(self) -> Color: ...
    def QC_Pt(self) -> Point3d: ...
    def QC_Vec(self) -> Vector3d: ...
    def QC_Complex(self) -> Complex: ...
    def QC_Matrix(self) -> Matrix: ...
    def QC_Interval(self) -> Interval: ...
    def QC_CompareTo(self, other: IGH_QuickCast) -> int: ...


class GH_GeometryGroupProxy:
    def __init__(self, owner: GH_GeometryGroup): ...


class GH_BoxProxy:
    def __init__(self, Value: GH_Box): ...
    def Construct(self) -> None: ...
    @property
    def ObjectID(self) -> str: ...
    @ObjectID.setter
    def ObjectID(self, Value: str) -> None: ...
    @property
    def Area(self) -> str: ...
    @property
    def Volume(self) -> str: ...
    @property
    def Origin(self) -> GH_Point3d_Wrapper: ...
    @property
    def XAxis(self) -> GH_Vector3d_Wrapper: ...
    @property
    def YAxis(self) -> GH_Vector3d_Wrapper: ...
    @property
    def ZAxis(self) -> GH_Vector3d_Wrapper: ...
    @property
    def XSize(self) -> GH_Interval_Wrapper: ...
    @property
    def YSize(self) -> GH_Interval_Wrapper: ...
    @property
    def ZSize(self) -> GH_Interval_Wrapper: ...


class GH_BrepProxy:
    def __init__(self, Value: GH_Brep): ...
    def Construct(self) -> None: ...
    def FromString(self, in: str) -> bool: ...
    @property
    def ObjectID(self) -> str: ...
    @ObjectID.setter
    def ObjectID(self, Value: str) -> None: ...
    @property
    def FaceCount(self) -> int: ...
    @property
    def EdgeCount(self) -> int: ...
    @property
    def VertexCount(self) -> int: ...
    @property
    def LoopCount(self) -> int: ...
    @property
    def Solid(self) -> bool: ...
    @property
    def Manifold(self) -> bool: ...
    @property
    def Area(self) -> str: ...
    @property
    def Volume(self) -> str: ...


class GH_CircleProxy:
    def __init__(self, nOwner: GH_Circle): ...
    def Construct(self) -> None: ...
    @property
    def Center(self) -> GH_Point3d_Wrapper: ...
    @property
    def Normal(self) -> GH_Vector3d_Wrapper: ...
    @property
    def Radius(self) -> str: ...
    @Radius.setter
    def Radius(self, Value: str) -> None: ...
    @property
    def Circumference(self) -> str: ...
    @Circumference.setter
    def Circumference(self, Value: str) -> None: ...
    @property
    def Area(self) -> str: ...
    @Area.setter
    def Area(self, Value: str) -> None: ...


class GH_CurveProxy:
    def __init__(self, owner: GH_Curve): ...
    def Construct(self) -> None: ...
    def FromString(self, in: str) -> bool: ...
    @property
    def ObjectID(self) -> str: ...
    @ObjectID.setter
    def ObjectID(self, Value: str) -> None: ...
    @property
    def Length(self) -> str: ...
    @property
    def Domain(self) -> str: ...
    @property
    def Planar(self) -> bool: ...
    @property
    def Linear(self) -> bool: ...
    @property
    def Closed(self) -> bool: ...
    @property
    def Periodic(self) -> bool: ...
    @property
    def Deformable(self) -> bool: ...
    @property
    def Type(self) -> str: ...


class GH_Interval2DProxy:
    def __init__(self, nValue: GH_Interval2D): ...
    @property
    def U0(self) -> float: ...
    @U0.setter
    def U0(self, Value: float) -> None: ...
    @property
    def U1(self) -> float: ...
    @U1.setter
    def U1(self, Value: float) -> None: ...
    @property
    def V0(self) -> float: ...
    @V0.setter
    def V0(self, Value: float) -> None: ...
    @property
    def V1(self) -> float: ...
    @V1.setter
    def V1(self, Value: float) -> None: ...
    @property
    def Area(self) -> float: ...
    @property
    def UMiddle(self) -> float: ...
    @property
    def ULength(self) -> float: ...
    @property
    def UIsIncreasing(self) -> bool: ...
    @UIsIncreasing.setter
    def UIsIncreasing(self, Value: bool) -> None: ...
    @property
    def UIsNormalized(self) -> bool: ...
    @UIsNormalized.setter
    def UIsNormalized(self, Value: bool) -> None: ...
    @property
    def VMiddle(self) -> float: ...
    @property
    def VLength(self) -> float: ...
    @property
    def VIsIncreasing(self) -> bool: ...
    @VIsIncreasing.setter
    def VIsIncreasing(self, Value: bool) -> None: ...
    @property
    def VIsNormalized(self) -> bool: ...
    @VIsNormalized.setter
    def VIsNormalized(self, Value: bool) -> None: ...


class GH_LineProxy:
    def __init__(self, nOwner: GH_Line): ...
    def Construct(self) -> None: ...
    @property
    def Ax(self) -> str: ...
    @Ax.setter
    def Ax(self, Value: str) -> None: ...
    @property
    def Ay(self) -> str: ...
    @Ay.setter
    def Ay(self, Value: str) -> None: ...
    @property
    def Az(self) -> str: ...
    @Az.setter
    def Az(self, Value: str) -> None: ...
    @property
    def Bx(self) -> str: ...
    @Bx.setter
    def Bx(self, Value: str) -> None: ...
    @property
    def By(self) -> str: ...
    @By.setter
    def By(self, Value: str) -> None: ...
    @property
    def Bz(self) -> str: ...
    @Bz.setter
    def Bz(self, Value: str) -> None: ...
    @property
    def Length(self) -> str: ...
    @Length.setter
    def Length(self, Value: str) -> None: ...


class RhMesherSettings_Proxy:
    def __init__(self, owner: GH_MeshingParameters): ...
    @property
    def IsParsable(self) -> bool: ...
    @property
    def SimplePlanes(self) -> bool: ...
    @SimplePlanes.setter
    def SimplePlanes(self, Value: bool) -> None: ...
    @property
    def AspectRatio(self) -> float: ...
    @AspectRatio.setter
    def AspectRatio(self, Value: float) -> None: ...
    @property
    def GridMinCount(self) -> int: ...
    @GridMinCount.setter
    def GridMinCount(self, Value: int) -> None: ...
    @property
    def GridMaxCount(self) -> int: ...
    @GridMaxCount.setter
    def GridMaxCount(self, Value: int) -> None: ...
    @property
    def Refine(self) -> bool: ...
    @Refine.setter
    def Refine(self, Value: bool) -> None: ...
    @property
    def Tolerance(self) -> float: ...
    @Tolerance.setter
    def Tolerance(self, Value: float) -> None: ...
    @property
    def AngleTolerance(self) -> float: ...
    @AngleTolerance.setter
    def AngleTolerance(self, Value: float) -> None: ...
    @property
    def MinEdgeLength(self) -> float: ...
    @MinEdgeLength.setter
    def MinEdgeLength(self, Value: float) -> None: ...
    @property
    def MaxEdgeLength(self) -> float: ...
    @MaxEdgeLength.setter
    def MaxEdgeLength(self, Value: float) -> None: ...
    @property
    def JaggedSeams(self) -> bool: ...
    @JaggedSeams.setter
    def JaggedSeams(self, Value: bool) -> None: ...


class GH_RectangleProxy:
    def __init__(self, nOwner: GH_Rectangle): ...
    def Construct(self) -> None: ...
    @property
    def Origin(self) -> GH_Point3d_Wrapper: ...
    @property
    def XAxis(self) -> GH_Vector3d_Wrapper: ...
    @property
    def YAxis(self) -> GH_Vector3d_Wrapper: ...
    @property
    def ZAxis(self) -> GH_Vector3d_Wrapper: ...
    @property
    def Width(self) -> GH_Interval_Wrapper: ...
    @property
    def Height(self) -> GH_Interval_Wrapper: ...
    @property
    def Area(self) -> str: ...
    @property
    def Circumference(self) -> str: ...


class GH_MeshProxy:
    def __init__(self, Value: GH_Mesh): ...
    def Construct(self) -> None: ...
    @property
    def ObjectID(self) -> str: ...
    @ObjectID.setter
    def ObjectID(self, Value: str) -> None: ...
    @property
    def FaceCount(self) -> int: ...
    @property
    def EdgeCount(self) -> int: ...
    @property
    def VertexCount(self) -> int: ...
    @property
    def Solid(self) -> bool: ...
    @property
    def Manifold(self) -> bool: ...
    @property
    def Area(self) -> str: ...
    @property
    def Volume(self) -> str: ...


class GH_MeshFaceProxy:
    def __init__(self, nValue: GH_MeshFace): ...
    @property
    def A(self) -> int: ...
    @A.setter
    def A(self, Value: int) -> None: ...
    @property
    def B(self) -> int: ...
    @B.setter
    def B(self, Value: int) -> None: ...
    @property
    def C(self) -> int: ...
    @C.setter
    def C(self, Value: int) -> None: ...
    @property
    def D(self) -> int: ...
    @D.setter
    def D(self, Value: int) -> None: ...
    @property
    def Kind(self) -> str: ...


class GH_PlaneProxy:
    def __init__(self, nOwner: GH_Plane): ...
    def Construct(self) -> None: ...
    @property
    def Origin(self) -> GH_Point3d_Wrapper: ...
    @property
    def XAxis(self) -> GH_Vector3d_Wrapper: ...
    @property
    def YAxis(self) -> GH_Vector3d_Wrapper: ...
    @property
    def ZAxis(self) -> GH_Vector3d_Wrapper: ...


class MaterialType:
    Shader = 0
    RhinoMaterial = 1
    XmlMaterial = 2
    RmtlMaterial = 3


class GH_Material_Proxy:
    def __init__(self, nValue: GH_Material): ...
    @property
    def Diffuse(self) -> Color: ...
    @Diffuse.setter
    def Diffuse(self, Value: Color) -> None: ...
    @property
    def Specular(self) -> Color: ...
    @Specular.setter
    def Specular(self, Value: Color) -> None: ...
    @property
    def Emission(self) -> Color: ...
    @Emission.setter
    def Emission(self, Value: Color) -> None: ...
    @property
    def Transparency(self) -> float: ...
    @Transparency.setter
    def Transparency(self, Value: float) -> None: ...
    @property
    def Shine(self) -> float: ...
    @Shine.setter
    def Shine(self, Value: float) -> None: ...
    @property
    def IsRDKMaterial(self) -> bool: ...


class GH_SurfaceProxy:
    def __init__(self, Value: GH_Surface): ...
    def Construct(self) -> None: ...
    def FromString(self, in: str) -> bool: ...
    @property
    def ObjectID(self) -> str: ...
    @ObjectID.setter
    def ObjectID(self, Value: str) -> None: ...
    @property
    def EdgeCount(self) -> int: ...
    @property
    def VertexCount(self) -> int: ...
    @property
    def LoopCount(self) -> int: ...
    @property
    def Solid(self) -> bool: ...
    @property
    def Trimmed(self) -> bool: ...
    @property
    def Planar(self) -> bool: ...
    @property
    def Singular(self) -> bool: ...
    @property
    def Closed(self) -> bool: ...
    @property
    def Periodic(self) -> bool: ...
    @property
    def Deformable(self) -> bool: ...
    @property
    def Area(self) -> str: ...
    @property
    def Volume(self) -> str: ...


class GH_PointProxy:
    def __init__(self, Value: GH_Point): ...
    def Construct(self) -> None: ...
    @property
    def Referenced(self) -> bool: ...
    @property
    def ReferenceID(self) -> str: ...
    @property
    def ReferenceType(self) -> str: ...
    @property
    def Distance(self) -> str: ...
    @Distance.setter
    def Distance(self, Value: str) -> None: ...
    @property
    def TParameterRatio(self) -> str: ...
    @TParameterRatio.setter
    def TParameterRatio(self, Value: str) -> None: ...
    @property
    def UVParameterRatio(self) -> GH_PointRefUV_Wrapper: ...
    @property
    def X(self) -> str: ...
    @X.setter
    def X(self, Value: str) -> None: ...
    @property
    def Y(self) -> str: ...
    @Y.setter
    def Y(self, Value: str) -> None: ...
    @property
    def Z(self) -> str: ...
    @Z.setter
    def Z(self, Value: str) -> None: ...


class GH_VectorProxy:
    def __init__(self, nOwner: GH_Vector): ...
    def Construct(self) -> None: ...
    @property
    def IsParsable(self) -> bool: ...
    def FromString(self, in: str) -> bool: ...
    def ToString(self) -> str: ...
    @property
    def X(self) -> str: ...
    @X.setter
    def X(self, Value: str) -> None: ...
    @property
    def Y(self) -> str: ...
    @Y.setter
    def Y(self, Value: str) -> None: ...
    @property
    def Z(self) -> str: ...
    @Z.setter
    def Z(self, Value: str) -> None: ...
    @property
    def Length(self) -> str: ...
    @Length.setter
    def Length(self, Value: str) -> None: ...
